# -*- coding: utf-8 -*-

default_config_resources = [
    'cleanit.yml',
    'cleanit-ocr.en.yml',
    'cleanit-no-spam.en.yml',
    'cleanit-tidy.en.yml',
    'cleanit-ocr.pt.yml',
    'cleanit-tidy.pt.yml',
    'cleanit-no-spam.pt.yml',
    'cleanit-ocr.yml',
    'cleanit-no-style.yml',
    'cleanit-no-sdh.yml',
    'cleanit-no-lyrics.yml',
    'cleanit-no-spam.yml',
    'cleanit-tidy.yml',
]
